@@ -33,6 +33,7 @@
 #include <signal.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/wait.h>
 #include <fcntl.h>
 #include <termios.h>
 #include <getopt.h>
@@ -69,7 +70,8 @@ static int signal_pipe[2];
 #define TTY_TX_MTU 65535
 
 int tty_init(int services, const gchar *root_path,
-		const gchar *capability, const gchar *devnode)
+		const gchar *capability, gboolean symlinks,
+		const gchar *devnode)
 {
 	struct server *server;
 	struct termios options;
@@ -91,6 +93,7 @@ int tty_init(int services, const gchar *root_path,
 
 	tcgetattr(fd, &options);
 	cfmakeraw(&options);
+	options.c_oflag &= ~ONLCR;
 	tcsetattr(fd, TCSANOW, &options);
 
 	arg = fcntl(fd, F_GETFL);
@@ -113,6 +116,7 @@ int tty_init(int services, const gchar *root_path,
 	server->devnode = g_strdup(devnode);
 	server->rx_mtu = TTY_RX_MTU;
 	server->tx_mtu = TTY_TX_MTU;
+	server->symlinks = symlinks;
 
 	io = g_io_channel_unix_new(fd);
 	g_io_channel_set_close_on_unref(io, TRUE);
@@ -158,6 +162,7 @@ static gboolean option_detach = TRUE;
 static gboolean option_debug = FALSE;
 
 static gchar *option_root = NULL;
+static gchar *option_root_setup = NULL;
 static gchar *option_capability = NULL;
 static gchar *option_devnode = NULL;
 
@@ -176,6 +181,8 @@ static GOptionEntry options[] = {
 				"Enable debug information output" },
 	{ "root", 'r', 0, G_OPTION_ARG_STRING, &option_root,
 				"Specify root folder location", "PATH" },
+	{ "root-setup", 'S', 0, G_OPTION_ARG_STRING, &option_root_setup,
+				"Root folder setup script", "SCRIPT" },
 	{ "symlinks", 'l', 0, G_OPTION_ARG_NONE, &option_symlinks,
 				"Enable symlinks on root folder" },
 	{ "capability", 'c', 0, G_OPTION_ARG_STRING, &option_capability,
@@ -217,7 +224,7 @@ static gboolean handle_signal(GIOChannel *io, GIOCondition cond,
 		tty_open_allowed = TRUE;
 		if (tty_needs_reinit)
 			tty_init(services, option_root, option_capability,
-							option_devnode);
+					option_symlinks, option_devnode);
 		break;
 	case SIGHUP:
 		debug("SIGHUP");
@@ -253,7 +260,46 @@ static int devnode_setup(void)
 		tty_open_allowed = FALSE;
 
 	return tty_init(services, option_root, option_capability,
-			option_devnode);
+			option_symlinks, option_devnode);
+}
+
+static gboolean is_dir(const char *dir) {
+	struct stat st;
+
+	if (stat(dir, &st) < 0) {
+		error("stat(%s): %s (%d)", dir, strerror(errno), errno);
+		return FALSE;
+	}
+
+	return S_ISDIR(st.st_mode);
+}
+
+static gboolean root_folder_setup(char *root, char *root_setup)
+{
+	gint status;
+	char *argv[3] = { root_setup, root, NULL };
+
+	if (is_dir(root))
+		return TRUE;
+
+	if (root_setup == NULL)
+		return FALSE;
+
+	debug("Setting up %s using %s", root, root_setup);
+
+	if (!g_spawn_sync(NULL, argv, NULL, 0, NULL, NULL, NULL, NULL,
+							&status, NULL)) {
+		error("Unable to execute %s", root_setup);
+		return FALSE;
+	}
+
+	if (WEXITSTATUS(status) != EXIT_SUCCESS) {
+		error("%s exited with status %d", root_setup,
+							WEXITSTATUS(status));
+		return FALSE;
+	}
+
+	return is_dir(root);
 }
 
 int main(int argc, char *argv[])
@@ -323,6 +369,14 @@ int main(int argc, char *argv[])
 	if (option_root == NULL)
 		option_root = g_strdup(DEFAULT_ROOT_PATH);
 
+	if (option_root[0] != '/') {
+		char *old_root = option_root, *home = getenv("HOME");
+		if (home) {
+			option_root = g_strdup_printf("%s/%s", home, old_root);
+			g_free(old_root);
+		}
+	}
+
 	if (option_capability == NULL)
 		option_capability = g_strdup(DEFAULT_CAP_FILE);
 
@@ -357,6 +411,11 @@ int main(int argc, char *argv[])
 	if (option_devnode)
 		devnode_setup();
 
+	if (!root_folder_setup(option_root, option_root_setup)) {
+		error("Unable to setup root folder %s", option_root);
+		exit(EXIT_FAILURE);
+	}
+
 	memset(&sa, 0, sizeof(sa));
 	sa.sa_handler = sig_term;
 	sigaction(SIGINT, &sa, NULL);
